@@ -17,11 +17,13 @@ from botocore import UNSIGNED, waiter, xform_name
 from botocore.args import ClientArgsCreator
 from botocore.auth import AUTH_TYPE_MAPS
 from botocore.awsrequest import prepare_request_dict
+from botocore.compress import maybe_compress_request
 from botocore.config import Config
 
 # Keep this imported.  There's pre-existing code that uses
 # "from botocore.client import UNSIGNED".
 from botocore.config import Config
+from botocore.credentials import RefreshableCredentials
 from botocore.discovery import (
     EndpointDiscoveryHandler,
     EndpointDiscoveryManager,
@@ -49,6 +51,7 @@ from botocore.utils import (
     CachedProperty,
     EventbridgeSignerSetter,
     S3ControlArnParamHandlerv2,
+    S3ExpressIdentityResolver,
     S3RegionRedirectorv2,
     ensure_boolean,
     get_service_module_name,
@@ -135,6 +138,7 @@ class ClientCreator(object):
         service_client = cls(**client_args)
         self._register_retries(service_client)
         self._register_s3_events(service_client, client_config, scoped_config)
+        self._register_s3express_events(client=service_client)
         self._register_s3_control_events(service_client)
         self._register_endpoint_discovery(
             service_client, endpoint_url, client_config
@@ -267,6 +271,18 @@ class ClientCreator(object):
             endpoint_url=endpoint_url
         ).register(client.meta.events)
 
+    def _register_s3express_events(
+        self,
+        client,
+        endpoint_bridge=None,
+        endpoint_url=None,
+        client_config=None,
+        scoped_config=None,
+    ):
+        if client.meta.service_model.service_name != 's3':
+            return
+        S3ExpressIdentityResolver(client, RefreshableCredentials).register()
+
     def _register_s3_events(self, client, client_config, scoped_config):
         if client.meta.service_model.service_name != 's3':
             return
@@ -656,9 +672,17 @@ class BaseClient(object):
             'auth_type': operation_model.auth_type,
         }
 
-        endpoint_url, additional_headers = self._resolve_endpoint_ruleset(
+        (
+            endpoint_url,
+            additional_headers,
+            properties,
+        ) = self._resolve_endpoint_ruleset(
             operation_model, api_params, request_context
         )
+        if properties:
+            # Pass arbitrary endpoint info with the Request
+            # for use during construction.
+            request_context['endpoint_properties'] = properties
         request_dict = self._convert_to_request_dict(
             api_params=api_params,
             operation_model=operation_model,
@@ -679,6 +703,9 @@ class BaseClient(object):
         if event_response is not None:
             http, parsed_response = event_response
         else:
+            maybe_compress_request(
+                self.meta.config, request_dict, operation_model
+            )
             apply_request_checksum(request_dict)
             http, parsed_response = self._make_request(
                 operation_model, request_dict, request_context)
@@ -788,6 +815,7 @@ class BaseClient(object):
         if self._ruleset_resolver is None:
             endpoint_url = self.meta.endpoint_url
             additional_headers = {}
+            endpoint_properties = {}
         else:
             endpoint_info = self._ruleset_resolver.construct_endpoint(
                 operation_model=operation_model,
@@ -796,6 +824,7 @@ class BaseClient(object):
             )
             endpoint_url = endpoint_info.url
             additional_headers = endpoint_info.headers
+            endpoint_properties = endpoint_info.properties
             # If authSchemes is present, overwrite default auth type and
             # signing context derived from service model.
             auth_schemes = endpoint_info.properties.get('authSchemes')
@@ -812,7 +841,7 @@ class BaseClient(object):
                 else:
                     request_context['signing'] = signing_context
 
-        return endpoint_url, additional_headers
+        return endpoint_url, additional_headers, endpoint_properties
 
     def get_paginator(self, operation_name):
         """Create a paginator for an operation.
